#!/usr/bin/perl

######
###### This executable will read from an input file and attempt to 
###### get the systemMetadata from a production cn machine
###### if successful it will attempt to get the object as well
###### It takes three arguments. The file to read the pids from.
###### the production CN node to verify against, and 
###### the dateSysMetadataModified that the objects should report. 
###### the date should be in the format of yyyy-dd-mm
###### and the yyyy,dd,mm may be a regular expression, though
###### when using regex make certain to surround the date is quotes on the
###### command line
######
###### example: verifyProdCNPopulationByDate.pl --file pids. in -node ORC --date "2013-07-\d{2}"
###### or even better
###### example: verifyProdCNPopulationByDate.pl --file pids. in -node ORC --date "\d{4}\-\d{2}\-\d{2}"
######
###### execute as verifyProdCNPopulationByDate.pl --file path_to_pid_file
######                 --node [ORC|UNM|UCSB]  --date yyyy-mm-dd
######
######

use strict;
use warnings;   
use Getopt::Long;
use LWP::UserAgent;
use LWP::Protocol::https;
use IO::Handle;

BEGIN { $| = 1 }

sub usage 
	{
	my $message = $_[0];
	if (defined $message && length $message) 
   		{
   		$message .= "\n" unless $message =~ /\n$/;
   		}
                                                                                                                                                                                                                                                                         
        my $command = $0;
        $command =~ s#^.*/##;

	print STDERR (
	      $message,
	      "usage: $command [--help] --file /path/to/file.in [-node node_of_cn]\n" ,
	      "\t--file must be the full path to the file to get pids from\n\n",
	      "\tthe file with the pids must contain a pid on a single line \n",
	      "\tfor each pid to verify\n",
	      "\t--node is the nodeId the evironment to verify\n",
	      "\tdefaults to the production boxes\n"
		);

	die("\n")
	}

sub getHttpsRequest
	{
	my ($url) = @_;
	my $rtn;
	my $ua = LWP::UserAgent->new;
	$ua->agent("d1ComponentRelease/1.0");
	$ua->timeout(30);
	my $data;
	# Create a request
	my $request = HTTP::Request->new(GET => $url);
	
	# Pass request to the user agent and get a response back
	my $response = $ua->request($request);
	
	# Check the outcome of the response
	if ($response->is_success) 
		{
		$rtn = $response->content;
	  	}
	else 
		{
		warn( $response->status_line . "\n");
		}
	return $rtn;
	}
	
main:	{
  my %Options;
  my $cn_url = "https://cn-unm-1.dataone.org/cn/v1/";
  my $file;
  my $node;
  my $date = '\d{4}\-\d{2}\-\d{2}';
  my @pidsToVerify;
  my $ok = GetOptions(\%Options, 
  	  "file=s",
  	  "node=s",
  	  "date=s",
	  'help') or usage("Invalid commmand line options.");
 
  usage("Have some Help") if ((defined $Options{help}) || !$ok);
  usage("Do you know what you are doing?") unless (defined($Options{file}));
  	  
  if (defined($Options{node}))
  	{
  	$node = $Options{node};
	if ($node =~ /ORC/i)
		{
		$cn_url = "https://cn-orc-1.dataone.org/cn/v1/";
		print "ORC\n";
		}
	elsif ($node =~ /UNM/i)
		{
  		$cn_url = "https://cn-unm-1.dataone.org/cn/v1/";
		print "UNM\n";
		}
	elsif ($node =~ /UCSB/i)
		{
		$cn_url = "https://cn-ucsb-1.dataone.org/cn/v1/";
		print "UCSB\n";
		}
	else
		{
		print "default\n";
		}
  	}
  if (defined($Options{date}))
  	{
  	$date = $Options{date};
  	}
  if (defined($Options{file}))
  	{
  	$file = $Options{file};
  	}
  open (VERIFY_FILE, $file)   or die "Couldn't open file: $!";
  while ( my $line = <VERIFY_FILE> ) 
	{
	chomp($line);
	$line =~ s/^\s+//;
	$line =~ s/\s+$//;
	
	unless ( !(defined($line)) || ($line =~ /^#/) || (length($line) <= 0) )
		{
		push (@pidsToVerify, $line);
		}
	}
  close VERIFY_FILE;

  foreach my $pid (@pidsToVerify)
  	{
  	print  "verify $pid meta ";
  	my $cn_meta_url = $cn_url . "meta/" . $pid;
  	my $content =  getHttpsRequest($cn_meta_url);
	if ($date !~ /\\\-/)
		{
		$date =~ s/\-/\\\-/g;
		print "$date\n";
		}
  	if (defined($content))
		{
		if ($content =~ /\<dateSysMetadataModified\>${date}T\d{2}:\d{2}:\d{2}.\d{3}\+\d{2}\:\d{2}\<\/dateSysMetadataModified\>/) 
			{
			print  " ok object ";
			my $cn_object_url = $cn_url . "object/" . $pid;
			$content =  getHttpsRequest($cn_object_url);
			if (defined($content))
				{
				if ($content =~ /https\:\/\/cn\.dataone\.org\/cn\/v1\/resolve\/$pid/) 
					{
					print  "found\n";
					}
				else
					{
					print "$pid object NOT FOUND\n";
					}
				}
			else
				{
				print "$pid no object found\n"

				}
			
			}
		else
			{
			print "$pid meta NOT FOUND\n";
			}
		}
	else
		{
		print "$pid no meta found\n"
		}
	}
  }
